@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 # Copyright, 2017, by Samuel G. D. Williams. <http://www.codeotaku.com>
 # 
 # Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -21,6 +23,8 @@
 require_relative 'buffer'
 require_relative 'generic'
 
+require 'async/semaphore'
+
 module Async
 	module IO
 		class Stream
@@ -42,8 +46,11 @@ module Async
 				@io = io
 				@eof = false
 				
-				@deferred = deferred
 				@pending = 0
+				# This field is ignored, but used to mean, try to buffer packets in a single iteration of the reactor.
+				# @deferred = deferred
+				
+				@writing = Async::Semaphore.new(1)
 				
 				# We don't want Ruby to do any IO buffering.
 				@io.sync = sync
@@ -53,12 +60,14 @@ module Async
 				
 				@read_buffer = Buffer.new
 				@write_buffer = Buffer.new
+				@drain_buffer = Buffer.new
 				
 				# Used as destination buffer for underlying reads.
 				@input_buffer = Buffer.new
 			end
 			
 			attr :io
+			
 			attr :block_size
 			
 			# Reads `size` bytes from the stream. If size is not specified, read until end of file.
@@ -66,7 +75,7 @@ module Async
 				return '' if size == 0
 				
 				if size
-					until @eof or @read_buffer.size >= size
+					until @eof or @read_buffer.bytesize >= size
 						# Compute the amount of data we need to read from the underlying stream:
 						read_size = size - @read_buffer.bytesize
 						
@@ -115,7 +124,7 @@ module Async
 				split_offset = pattern.bytesize - 1
 				
 				until index = @read_buffer.index(pattern, offset)
-					offset = @read_buffer.size - split_offset
+					offset = @read_buffer.bytesize - split_offset
 					
 					offset = 0 if offset < 0
 					
@@ -135,20 +144,36 @@ module Async
 				end
 			end
 			
+			def gets(separator = $/, **options)
+				read_until(separator, **options)
+			end
+			
+			# Flushes buffered data to the stream.
+			def flush
+				return if @write_buffer.empty?
+				
+				@writing.acquire do
+					# Flip the write buffer and drain buffer:
+					@write_buffer, @drain_buffer = @drain_buffer, @write_buffer
+					
+					begin
+						@io.write(@drain_buffer)
+					ensure
+						# If the write operation fails, we still need to clear this buffer, and the data is essentially lost.
+						@drain_buffer.clear
+					end
+				end
+			end
+			
 			# Writes `string` to the buffer. When the buffer is full or #sync is true the
 			# buffer is flushed to the underlying `io`.
 			# @param string the string to write to the buffer.
 			# @return the number of bytes appended to the buffer.
 			def write(string)
-				if @write_buffer.empty? and string.bytesize >= @block_size
-					@io.write(string)
-				else
-					@write_buffer << string
-					
-					if @write_buffer.size >= @block_size
-						@io.write(@write_buffer)
-						@write_buffer.clear
-					end
+				@write_buffer << string
+				
+				if @write_buffer.bytesize >= @block_size
+					flush
 				end
 				
 				return string.bytesize
@@ -161,37 +186,9 @@ module Async
 				return self
 			end
 			
-			# Flushes buffered data to the stream.
-			def flush(deferred: @deferred)
-				unless @write_buffer.empty?
-					if deferred and task = Task.current?
-						if @pending.zero?
-							@pending += 1
-							task.yield
-						else
-							@pending += 1
-							# task.yield
-							return
-						end
-						
-						Async.logger.debug(self) {"Flushing #{@pending} writes (#{@write_buffer.bytesize} bytes)..."}
-						
-						@pending = 0
-					else
-						Async.logger.debug(self) {"Flushing immediate write (#{@write_buffer.bytesize} bytes)..."}
-					end
-					
-					drain_write_buffer
-				end
-			end
-			
-			def gets(separator = $/, **options)
-				read_until(separator, **options)
-			end
-			
-			def puts(*args, separator: $/)
-				args.each do |arg|
-					@write_buffer << arg << separator
+			def puts(*arguments, separator: $/)
+				arguments.each do |argument|
+					@write_buffer << argument << separator
 				end
 				
 				flush
@@ -210,7 +207,7 @@ module Async
 			end
 			
 			def close_write
-				drain_write_buffer
+				flush
 			ensure
 				@io.close_write
 			end
@@ -220,7 +217,7 @@ module Async
 				return if @io.closed?
 				
 				begin
-					drain_write_buffer
+					flush
 				rescue
 					# We really can't do anything here unless we want #close to raise exceptions.
 				ensure
@@ -230,13 +227,13 @@ module Async
 			
 			# Returns true if the stream is at file which means there is no more data to be read.
 			def eof?
-				return false unless @read_buffer.empty?
-				
-				if !@eof
-					fill_read_buffer
+				if !@read_buffer.empty?
+					return false
+				elsif @eof
+					return true
+				else
+					return @io.eof?
 				end
-				
-				return @eof
 			end
 			
 			alias eof eof?
@@ -250,14 +247,6 @@ module Async
 			
 			private
 			
-			def drain_write_buffer
-				unless @write_buffer.empty?
-					# Async.logger.debug(self, name: "write") {@write_buffer.inspect}
-					@io.write(@write_buffer)
-					@write_buffer.clear
-				end
-			end
-			
 			# Fills the buffer from the underlying stream.
 			def fill_read_buffer(size = @block_size)
 				# We impose a limit because the underlying `read` system call can fail if we request too much data in one go.
@@ -265,6 +254,9 @@ module Async
 					size = @maximum_read_size
 				end
 				
+				# This effectively ties the input and output stream together.
+				flush
+				
 				if @read_buffer.empty?
 					if @io.read_nonblock(size, @read_buffer, exception: false)
 						# Async.logger.debug(self, name: "read") {@read_buffer.inspect}
@@ -274,6 +266,7 @@ module Async
 					if chunk = @io.read_nonblock(size, @input_buffer, exception: false)
 						@read_buffer << chunk
 						# Async.logger.debug(self, name: "read") {@read_buffer.inspect}
+						
 						return true
 					end
 				end
@@ -291,7 +284,7 @@ module Async
 				
 				result = nil
 				
-				if size.nil? or size >= @read_buffer.size
+				if size.nil? or size >= @read_buffer.bytesize
 					# Consume the entire read buffer:
 					result = @read_buffer
 					@read_buffer = Buffer.new
